import { Directive, ElementRef, forwardRef, Input, Renderer2 } from "@angular/core";
import { NG_ASYNC_VALIDATORS, NG_VALUE_ACCESSOR } from "@angular/forms";
import "rxjs/add/observable/empty";
import "rxjs/add/observable/forkJoin";
import "rxjs/add/observable/fromEvent";
import "rxjs/add/observable/fromPromise";
import "rxjs/add/observable/interval";
import "rxjs/add/observable/of";
import "rxjs/add/operator/map";
import "rxjs/add/operator/mergeMap";
import "rxjs/add/operator/take";
import { Observable } from "rxjs/Observable";
import { ReplaySubject } from "rxjs/ReplaySubject";
var FileInputAccessor = (function () {
    function FileInputAccessor(_renderer, _elementRef) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this.onChange = function (_) {
        };
        this.onTouched = function () {
        };
        this.validator = this.generateAsyncValidator();
    }
    FileInputAccessor.prototype.ngOnChanges = function (changes) {
        if (changes.fileList && changes.fileList.currentValue.length == 0) {
            this._renderer.setProperty(this._elementRef.nativeElement, 'value', []);
        }
    };
    FileInputAccessor.prototype.writeValue = function (value) {
        var normalizedValue = value == null ? '' : value;
        this._renderer.setProperty(this._elementRef.nativeElement, 'value', normalizedValue);
    };
    FileInputAccessor.prototype.registerOnChange = function (fn) {
        this.onChange = this.onChangeGenerator(fn);
    };
    FileInputAccessor.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    FileInputAccessor.prototype.setDisabledState = function (isDisabled) {
        throw new Error("Method not implemented.");
    };
    FileInputAccessor.prototype.validate = function (c) {
        return this.validator(c);
    };
    FileInputAccessor.prototype.generateAsyncValidator = function () {
        var _this = this;
        return function (c) {
            if (!c.value || !c.value.length)
                return Observable.of({});
            var errors = {};
            var loaders = [];
            var _loop_1 = function (f) {
                if (_this.maxSize && _this.maxSize < f.size) {
                    f.errors['fileSize'] = true;
                    errors['fileSize'] = true;
                }
                if (f.isImg && (_this.maxWidth || _this.maxHeight)) {
                    loaders.push(f.imgLoadReplay
                        .take(1)
                        .map(function (e) {
                        if (_this.maxWidth && f.imgWidth > _this.maxWidth) {
                            f.errors['imageWidth'] = true;
                            errors['imageWidth'] = true;
                        }
                        if (_this.maxHeight && f.imgHeight > _this.maxHeight) {
                            f.errors['imageHeight'] = true;
                            errors['imageHeight'] = true;
                        }
                        return e;
                    }));
                }
                if (!_this.extPattern && !_this.typePattern)
                    return "continue";
                var extP = _this.generateRegExp(_this.extPattern);
                var typeP = _this.generateRegExp(_this.typePattern);
                if (extP && !extP.test(f.name)) {
                    f.errors['fileExt'] = true;
                    errors['fileExt'] = true;
                }
                if (typeP && !typeP.test(f.type)) {
                    f.errors['fileType'] = true;
                    errors['fileType'] = true;
                }
            };
            for (var _i = 0, _a = c.value; _i < _a.length; _i++) {
                var f = _a[_i];
                _loop_1(f);
            }
            if (loaders.length) {
                return Observable
                    .forkJoin.apply(Observable, loaders).map(function (loadResults) {
                    return errors;
                });
            }
            return Observable.of(errors);
        };
    };
    ;
    FileInputAccessor.prototype.onChangeGenerator = function (fn) {
        var _this = this;
        return function (files) {
            var fileArr = [];
            if (!files || !files.length) {
                _this._renderer.setProperty(_this._elementRef.nativeElement, 'value', []);
                return fn(fileArr);
            }
            for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
                var f = files_1[_i];
                if (_this.addMeta && FileReader) {
                    var fr = new FileReader();
                    _this.generateFileMeta(f, fr);
                }
                f.errors = {};
                fileArr.push(f);
            }
            return fn(fileArr);
        };
    };
    ;
    FileInputAccessor.prototype.generateRegExp = function (pattern) {
        if (!pattern)
            return null;
        if (pattern instanceof RegExp) {
            return new RegExp(pattern);
        }
        else if (typeof pattern === 'string') {
            return new RegExp(pattern, 'ig');
        }
        else if (pattern instanceof Array) {
            return new RegExp("(" + pattern.join('|') + ")", 'ig');
        }
        return null;
    };
    FileInputAccessor.prototype.generateFileMeta = function (f, fr) {
        if (f.type.match(/text.*/)) {
            f.textLoadReplay = this.setText(f, fr);
        }
        else if (f.type.match(/image.*/)) {
            f.imgLoadReplay = this.setImage(f, fr);
        }
    };
    FileInputAccessor.prototype.setImage = function (f, fr) {
        f.isImg = true;
        var img = new Image();
        var imgLoadObs = Observable.fromEvent(img, 'load')
            .take(1)
            .map(function (e) {
            f.imgHeight = img.height;
            f.imgWidth = img.width;
            return e;
        });
        var frLoadObs = Observable.fromEvent(fr, 'load')
            .take(1)
            .map(function (e) {
            f.imgSrc = fr.result;
            img.src = fr.result;
            return e;
        });
        var onloadReplay = new ReplaySubject(1);
        Observable
            .forkJoin(imgLoadObs, frLoadObs)
            .subscribe(onloadReplay);
        fr.readAsDataURL(f);
        return onloadReplay;
    };
    FileInputAccessor.prototype.setText = function (f, fr) {
        var onloadReplay = new ReplaySubject(1);
        Observable.fromEvent(fr, 'load')
            .map(function (e) {
            f.textContent = fr.result;
            return [e];
        })
            .subscribe(onloadReplay);
        fr.readAsText(f);
        return onloadReplay;
    };
    return FileInputAccessor;
}());
export { FileInputAccessor };
FileInputAccessor.decorators = [
    { type: Directive, args: [{
                selector: 'input[type=file][formControl],input[type=file][formControlName],input[type=file][ngModel]',
                host: { '(change)': 'onChange($event.target.files)', '(blur)': 'onTouched()' },
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(function () { return FileInputAccessor; }),
                        multi: true
                    },
                    {
                        provide: NG_ASYNC_VALIDATORS,
                        useExisting: forwardRef(function () { return FileInputAccessor; }),
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
FileInputAccessor.ctorParameters = function () { return [
    { type: Renderer2, },
    { type: ElementRef, },
]; };
FileInputAccessor.propDecorators = {
    'extPattern': [{ type: Input, args: ['allowedExt',] },],
    'typePattern': [{ type: Input, args: ['allowedTypes',] },],
    'maxSize': [{ type: Input, args: ['size',] },],
    'addMeta': [{ type: Input, args: ['withMeta',] },],
    'maxHeight': [{ type: Input, args: ['maxHeight',] },],
    'maxWidth': [{ type: Input, args: ['maxWidth',] },],
};
//# sourceMappingURL=file-input-accessor.js.map